# -*- coding: utf-8 -*-

from django import forms
from ckeditor_uploader.widgets import CKEditorUploadingWidget
from alapage.models import Page
from alapage.conf import CODE_MODE, CODEMIRROR_KEYMAP
if CODE_MODE is True:
    from codemirror2.widgets import CodeMirrorEditor


class PageAdminForm(forms.ModelForm):

    if CODE_MODE is False:
        content = forms.CharField(widget=CKEditorUploadingWidget())
    else:
        content = forms.CharField(
            widget=CodeMirrorEditor(options={
                'mode': 'htmlmixed',
                'width': '1170px',
                'indentWithTabs': 'true',
                #'indentUnit' : '4',
                'lineNumbers': 'true',
                'autofocus': 'true',
                #'highlightSelectionMatches': '{showToken: /\w/, annotateScrollbar: true}',
                'styleActiveLine': 'true',
                'autoCloseTags': 'true',
                'keyMap': CODEMIRROR_KEYMAP,
                'theme': 'blackboard',
                #'fullScreen':'true',
            },
                script_template='codemirror2/codemirror_script_alapage.html',
                modes=['css', 'xml', 'javascript', 'htmlmixed'],
            )

        )
    content.required = False
    content.label = ""

    class Meta:
        model = Page
        exclude = ('edited', 'created', "editor")



# -*- coding: utf-8 -*-

from django.test import TestCase, override_settings
from django.contrib.admin.sites import AdminSite
from django.contrib.auth.models import User
from alapage.models import Page
from alapage.admin import PageAdmin, PageAdminForm
from django.core.urlresolvers import reverse

# models test
@override_settings(USE_JSSOR=True)
class PageTest(TestCase):
    
    def create_page(self, url='/mypage/', layout='xs-12', published=True, template_name=''):
        page = Page.objects.create(url=url, layout=layout, published=published, template_name=template_name)
        return page
    
    def test_page_creation(self):
        page=self.create_page()
        self.assertTrue(isinstance(page, Page))
        self.assertEqual(page.url, '/mypage/')
        self.assertEqual(page.layout, 'xs-12')
        self.assertTrue(page.published)
      
    #~ client tests
    def test_404_page(self):
        response = self.client.get(reverse('page-view', kwargs={"url":"/mypage/"}))
        self.assertEqual(response.status_code, 404)
        
    def test_page(self):
        page=self.create_page()
        response = self.client.get(reverse('page-view', kwargs={"url":"/mypage/"}))
        self.assertEqual(response.status_code, 200)
        self.assertTrue('page' in response.context)
        self.assertTrue('layout' in response.context)
        self.assertTrue('layout_path' in response.context)
        self.assertTrue(page.published)
        self.assertTemplateUsed(response, 'alapage/default.html')
        
    def test_trailing_slash(self):
        self.create_page(url='/mypage/')
        response = self.client.get('/mypage/')
        self.assertEqual(response.status_code, 200)
        
    def page_not_published(self):
        page=self.create_page(url='/mypage/', published=False)
        response = self.client.get('/mypage/')
        self.assertEqual(response.status_code, 200)
        self.assertFalse(page.published)
        
    def test_template_name(self):
        page=self.create_page(url='/mypage/', template_name='alapage/layouts/bottom.html')
        response = self.client.get(reverse('page-view', kwargs={"url":"/mypage/"}))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(page.template_name, 'alapage/layouts/bottom.html')
    
    #~ admin
    def test_admin_form(self):
        admin = PageAdmin(Page, AdminSite())
        username = 'test_user'
        pwd = 'secret'
        self.user = User.objects.create_superuser(username, 'myemail@test.com', pwd)
        self.client.login(username=username, password=pwd)
        form_data = {'url':'/mypage/', 'content':'Lorem ipsum','title':'Title'}
        form = PageAdminForm(data=form_data)
        self.assertTrue(form.is_valid())
        form.save(commit=True)
        page=Page.objects.get(url='/mypage/')
        response = self.client.get('/admin/')
        request = response.wsgi_request
        admin.save_model(request, page, form, True)
        self.assertTrue(page.editor ,self.user)


# -*- coding: utf-8 -*-

from django.conf.urls import url
from alapage.views import HomepageView, PageView, PagesmapView


urlpatterns = []

urlpatterns.append(
    url(r'^sitemap/$', PagesmapView.as_view(), name="alapage-map"))
#urlpatterns.append(url(r'^alapage/wizard/$', PageWizardView.as_view(), name="alapage-wizard"))
#urlpatterns.append(url(r'^alapage/wizard/post/$', AddPagePostView.as_view(), name="alapage-wizard-post"))
urlpatterns.append(
    url(r'^(?P<url>.*?)$', PageView.as_view(), name="page-view"))
urlpatterns.append(url(r'^', HomepageView.as_view(), name="home-view"))



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import Group
from mptt.models import TreeForeignKey, MPTTModel
from alapage.conf import USER_MODEL


class Seo(models.Model):
    seo_description = models.CharField(max_length=256, null=True, blank=True, verbose_name=_(
        u'SEO: description'), help_text=_(u'Short description of the page content'))
    seo_keywords = models.CharField(max_length=120, null=True, blank=True, verbose_name=_(
        u'SEO: keywords'), help_text=_(u'List of keywords separated by commas'))

    class Meta:
        abstract = True
        verbose_name = _(u'SEO')


class Page(MPTTModel, Seo):
    url = models.CharField(_(u'Url'), max_length=180, db_index=True)
    title = models.CharField(_(u'Title'), max_length=200)
    content = models.TextField(_(u'Content'), blank=True)
    template_name = models.CharField(_(u'Template name'), max_length=120, blank=True,
                                     help_text=_(
                                         u'If no template name is provided "alapage/default.html" will be used.')
                                     )
    registration_required = models.BooleanField(
        _(u'Registration required'),
        help_text=_(
            u"If this is checked, only logged-in users will be able to view the page."),
        default=False,
    )
    parent = TreeForeignKey('self', null=True, blank=True,
                            related_name=u'children', verbose_name=_(u'Parent page'))
    edited = models.DateTimeField(
        editable=False, null=True, auto_now=True, verbose_name=_(u'Edited'))
    created = models.DateTimeField(
        editable=False, null=True, auto_now_add=True, verbose_name=_(u'Created'))
    editor = models.ForeignKey(USER_MODEL, editable=False, related_name='+',
                               null=True, on_delete=models.SET_NULL, verbose_name=_(u'Edited by'))
    published = models.BooleanField(default=True, verbose_name=_(u'Published'))
    staff_only = models.BooleanField(
        default=False, verbose_name=_(u'Staff only'))
    superuser_only = models.BooleanField(
        default=False, verbose_name=_(u'Superuser only'))
    groups_only = models.ManyToManyField(
        Group, blank=True, verbose_name=_(u'Reserved to some groups'))
    users_only = models.ManyToManyField(
        USER_MODEL, blank=True, verbose_name=_(u'Reserved to some users'))
    # page caracteristics used to tminimize the select related queries
    is_reserved_to_groups = models.BooleanField(
        default=False, verbose_name=_(u'Reserved to groups'))
    is_reserved_to_users = models.BooleanField(
        default=False, verbose_name=_(u'Reserved to users'))

    class Meta:
        verbose_name = _(u'Page')
        verbose_name_plural = _(u'Page')
        ordering = ['url']
        permissions = (
            ("can_change_page_permissions", "Can change page permissions"),
        )

    def get_absolute_url(self):
        return self.url

    def __str__(self):
        return self.title



from django.utils.translation import ugettext_lazy as _
from django.apps import AppConfig


class AlapageConfig(AppConfig):
    name = "alapage"
    verbose_name = _(u"Alapage")
    
    def ready(self):
        pass
        
        


from django.conf import settings
from django.contrib.auth.models import User


USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', User)

USE_REVERSION = getattr(settings, 'ALAPAGE_USE_REVERSION',
                        "reversion" in settings.INSTALLED_APPS)

BASE_TEMPLATE_PATH = getattr(
    settings, 'ALAPAGE_BASE_TEMPLATE_PATH', 'base.html')

CODE_MODE = getattr(settings, 'ALAPAGE_CODE_MODE', True)
CODEMIRROR_KEYMAP = getattr(settings, 'ALAPAGE_CODEMIRROR_KEYMAP', 'default')



from setuptools import setup, find_packages


version = __import__('alapage').__version__

setup(
    name='django-alapage',
    packages=find_packages(),
    include_package_data=True,
    version=version,
    description='Page management app for Django',
    author='synw',
    author_email='synwe@yahoo.com',
    url='https://github.com/synw/django-alapage',
    download_url='https://github.com/synw/django-alapage/releases/tag/' + version,
    keywords=['django', 'page management'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
    ],
    install_requires=[
        'Pillow',
        'pytz',
        'django-ckeditor',
        'django-codemirror2',
        "django-mptt",
        "django-mptt-graph",
    ],
    zip_safe=False
)



from __future__ import print_function
from django.core.management.base import BaseCommand, CommandError
from alapage.models import Page

class Command(BaseCommand):
    help = 'Creates a homepage'

    def handle(self, *args, **options):
        content = ""
        #~ check if home exists
        home_exists = Page.objects.filter(url='/').exists()
        #~ create page
        if not home_exists:
            Page.objects.create(url='/', title='Home', content=content)
            print("Homepage created")
        else:
            print("The homepage already exists with root url")
        return